#!/bin/bash
# Check which links are alive, do it by also checking the drops
# $1: config file
# $2: "fast", optional, do fast check
# Can be called by cron:
#*/5 * * * * /script_path/mpath_connmark_check.sh /conf_path/mpath_connmark.conf &> /dev/null

cfg="$1"
fast="$2"

# Alive links
mask=-1

if ! [ -e "$cfg" ]
then
	echo "Missing configuration file: $cfg"
	exit 1
fi

dp=""
cd "`dirname "$0"`" && dp="`pwd`"
. "$cfg"

check_ip() {
	local i
	local f=0
	local ip="$1"
	local nok=10

	[ "$fast" != "" ] && nok=3

	# Check up to 30 seconds
	for ((i = 0; i < 30; i++))
	do
		if ping -c 1 "$ip" &> /dev/null
		then
			[ "$[$i&1]" -ne 0 ] && sleep 1
		else
			let f+=1
		fi
		# first 10 successful retries => good
		[ "$i" -ge $nok -a "$f" -eq 0 ] && return 0
		# At least 3 failures and 20% errors
		[ "$f" -ge 3 -a "$[$f*5]" -ge "$i" ] && return 1
	done
	# 16% errors
	[ "$f" -ge 5 ] && return 1
	return 0
}

# Prefer previous state
[ -e "$s" ] && . $s

old_mask="$mask"

mask=0
for ((lnk=0; lnk < ${#LINKS[@]}; lnk++))
do
	[ "${LINKS[$lnk]}" != y ] && continue
	ip="${CHKGW[$lnk]}"
	[ "$ip" = "" ] && continue
	bits="$[1 << $lnk]"
	if check_ip "$ip"
	then
		# Inhibited link will look like a dead one
		if ! [ -e "$dis$lnk" ]
		then
			mask="$[$mask+$bits]"
		fi
	fi
done

if [ "$mask" -lt 0 -o "$mask" -ne "$old_mask"  ]
then
	if [ "$c" != "" ]
	then
		# Optionally, reduce the mask of alive links with
		# the links from first priority group containing alive links
		mask="`$c "$mask" "$old_mask" "$cfg"`"
		ip route flush cache
	fi
	[ "$mask" != "" ] && echo "mask=$mask" > "$s"
fi

exit 0

